!+
! Program:  epevents/test/checkoutput
!
! Authors:  Matthias Ehle, Michael Freyberg - XMM-SSC-MPE
!          
! Description: Tests output and functionality of epevents
! 
!- 

program checkoutput_IM

use dal
use errorhandling 
use eventAttributes

implicit none

type(DataSetT)  :: inevents
type(TableT)    :: eventstable, expotable, bpixtable
type(ColumnT)   :: detXCol, detYCol, piCol, &
    		   pat_typCol, pat_indCol, &
		   flagCol
		   
integer(kind=Int32),dimension(:), pointer :: flag
integer(kind=Int16),dimension(:), pointer :: detX, detY, pi, &
					     pat_typ, pat_ind
					     
integer (kind=Int32)    :: xmin, xmax, ymin, ymax, nrows
integer,parameter       :: itest = 2

integer (kind=Int32)    :: i  ! , n2 = 82
logical (kind=bool)     :: lerror = .false.

character(len=16),dimension(5)            :: list = (/"events_no.dat", &
                                                      "events_IM.dat", &
                                                      "events_TI.dat", &		   
                                                      "events_BU.dat", &
                                                      "events_XX.dat"/)			   

character(len=20)                         :: expname, readoutmode


  if (itest == 1 .or. itest == 2) readoutmode = 'IMAGING'
  if (itest == 3) readoutmode = 'TIMING'
  if (itest == 4) readoutmode = 'BURST'
  if (itest == 5) readoutmode = 'IMAGING'
  
  write (*,*) 'testing epevents ',trim(readoutmode),' mode output:'

!----------------------------------------
write (*,*) 'testing ',list(itest),' ...'
!----------------------------------------

inevents = dataSet(list(itest), read)

!-----------------------------------

eventstable = table(inevents,"EVENTS")

if (.not.hasColumn(eventstable,"TIME")    .or. &
    .not.hasColumn(eventstable,"FRAME")   .or. &
    .not.hasColumn(eventstable,"RAWX")    .or. &
    .not.hasColumn(eventstable,"RAWY")    .or. &
    .not.hasColumn(eventstable,"DETX")    .or. &
    .not.hasColumn(eventstable,"DETY")    .or. &    
    .not.hasColumn(eventstable,"PHA")     .or. &
    .not.hasColumn(eventstable,"PHA_CTI") .or. &
!   .not.hasColumn(eventstable,"PHA_GAIN").or. &
    .not.hasColumn(eventstable,"PI")      .or. &
    .not.hasColumn(eventstable,"CCDNR")   .or. &
    .not.hasColumn(eventstable,"FLAG")    .or. &
    .not.hasColumn(eventstable,"PATTERN") .or. &
    .not.hasColumn(eventstable,"PAT_ID")  .or. &
    .not.hasColumn(eventstable,"PAT_TYP") .or. &
    .not.hasColumn(eventstable,"PAT_IND") .or. &
    .not.hasColumn(eventstable,"PAT_ORI") .or. & 
    .not.hasColumn(eventstable,"PAT_SEQ")) &
    call error("noEvColumn", &
                        "Events extension: column absent or incorrectly named")

if (.not.hasAttribute(eventstable,"SCR_88")) &
      call error("noSCR_88", &
                 "events extension: SCR_88 event attribute keyword missing")

if (.not.hasAttribute(eventstable,"XMMEA_16")) &
      call error("noXMMEA_16", &
                 "events extension: OUT_OF_FOV event attribute keyword missing")

if (.not.hasAttribute(eventstable,"FRAMECTI")) &
      call error("noFRAMECTI", &
                 "events extension: FRAMECTI keyword missing")

if (.NOT. booleanAttribute(eventstable,"FRAMECTI")) &
      call error("falseFRAMECTI", &
                 "events extension: FRAMECTI keyword is FALSE instead of TRUE")

write(expname,'(a)') "EXPOSURE"

if (hasBlock(inevents,expname)) then
    expotable = table(inevents,expname)

    if (.not.hasColumn(expotable,"TIME")     .or. &
        .not.hasAttribute(expotable,"TIMEDEL")  .or. &
        .not.hasColumn(expotable,"FRACEXP")) call error("noExpoColumn", &
                      "exposure extension: column absent or incorrectly named")
    if (.not.hasAttribute(expotable,"SCR_88")) &
      call error("noSCR_88", &
                 "exposure extension: SCR_88 event attribute keyword missing")

endif
  
if (hasBlock(inevents,"BADPIX")) then
    bpixtable = table(inevents,"BADPIX")

    if (.not.hasColumn(bpixtable,"RAWX")    .or. &
        .not.hasColumn(bpixtable,"RAWY")    .or. &
        .not.hasColumn(bpixtable,"TYPE")    .or. &
        .not.hasColumn(bpixtable,"YEXTENT") .or. &
        .not.hasColumn(bpixtable,"BADFLAG")) call error("noBadColumn", &
                      "badpix extension: column absent or incorrectly named")
endif

nrows = numberofRows(eventstable)

if (itest > 1) then

  !-------------------------------------------
  write(*,*) 'testing detector coordinates...'
  !-------------------------------------------

  detXCol = column(eventstable, "DETX", read)
  detYCol = column(eventstable, "DETY", read)
  flagCol = column(eventstable, "FLAG", read)

  detX => int16Data(detXCol)
  detY => int16Data(detYCol)
  flag => int32Data(flagCol)

  xmin = int16Attribute(detXCol,"TLMIN")
  xmax = int16Attribute(detXCol,"TLMAX")
  ymin = int16Attribute(detYCol,"TLMIN")
  ymax = int16Attribute(detYCol,"TLMAX")

  do i = 0,  nrows-1
   if (flag(i) == 0) then
     if (detX(i) < xmin .or. detX(i) > xmax) then
       write(errstr,*) "detX(",i,") out of range: ",detX(i),&
          ' (range = [',xmin,',',xmax,'])'
       call error("wrongDetXCoords")
     endif
     if (detY(i) < ymin .or. detY(i) > ymax) then
       write(errstr,*) "detY(",i,") out of range: ",detY(i),&
          ' (range = [',ymin,',',ymax,'])'
       call error("wrongDetYCoords")
     endif
    endif    
  enddo

  pat_typCol = column(eventstable, "PAT_TYP", read)
  pat_indCol = column(eventstable, "PAT_IND", read)
  piCol      = column(eventstable, "PI", read)

  pat_typ => int16Data(pat_typCol)
  pat_ind => int16Data(pat_indCol)
  pi      => int16Data(piCol)

  !--------------------------------------
  write(*,*) 'testing MIP recognition...'
  !--------------------------------------
  
  do i = 0, nrows-1
    if (iand(flag(i),EVATT_COSMIC_RAY) > 0) then 
      if (pat_typ(i) < 1 .and. pi(i) > 0) then
        if (.not.isNull(piCol,i)) then
          write(errstr,*) "Error in MIP recognition: pat_typ(",i,")=",pat_typ(i), &
                          " pi(",i,")=",pi(i)
          call error("wrongMIP")
    	endif
      endif  				         
    endif
  
  enddo

  !----------------------------------------------------
  write(*,*) 'testing recombination of split events...'
  !----------------------------------------------------

  do i = 0, nrows-1
    if (iand(flag(i),EVATT_SECONDARY) > 0 .and. .not.isNull(piCol,i) .or. &
        iand(flag(i),EVATT_SECONDARY) == 0 .and. pi(i) == 0) then 
      write(errstr,*) "Incorrect recombination of split events: pat_ind(",i,&
                      ")=",pat_ind(i)," pi(",i,")=",pi(i)
      call error("wrongRecombine")
    endif 
  enddo

  !-------------------------------------
  write(*,*) 'testing PI calibration...'
  !-------------------------------------

  do i = 0, nrows-1
    if (pat_ind(i) >= 0 .and. flag(i) == 0) then
      if (itest == 1) then
        if (pi(i) < 5578 .or. pi(i) > 6051) then
          write(errstr,*) "Incorrect PI calibration: pi(",i,")=",pi(i), &
            ' DETX,DETY,PAT_TYP,FLAG =',detx(i),dety(i),pat_typ(i),flag(i) 
          call message(AppMsg,Sparse)
          lerror = .true.
        endif
      endif
      if (itest == 2) then
        if (pi(i) < -345 .or. pi(i) > 15652) then
          write(*,*) "Incorrect PI calibration: pi(",i,")=",pi(i), &
            ' DETX,DETY,PAT_TYP,FLAG =',detx(i),dety(i),pat_typ(i),flag(i)
          lerror = .true.
        endif
      endif  
    endif
  enddo
  if (lerror) then
    if (itest == 2) then
      do i = 0,nrows-1
        write(errstr,*) 'I,DETX,DETY,PAT_TYP,FLAG,PI = ', &
              i,detx(i),dety(i),pat_typ(i),flag(i),pi(i)
        call message(AppMsg,Sparse)
      enddo 
    endif
    call error("wrongPI"," ")
  endif
 !-----------------------------------
  
endif

call release(inevents)

end
